# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#INITIAL NAME : SetStop.py

import argparse
import threading
from time import sleep

import os
import sys

from FXHelperTransact import print_jsonl_message

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtutils import jgtconstants as constants

from jgtutils import  jgtcommon
from jgtutils.FXTransact import FXTrade,FXTrades
from jgtutils.FXTransact import FXTransactDataHelper as fxtdh

from forexconnect import fxcorepy, ForexConnect, Common, EachRowListener

import common_samples

str_account = None
str_instrument = None
str_stop = None
str_trade_id = None
pips_flag=False
quiet=True
fxtrade:FXTrade=None
fxtrades:FXTrades=FXTrades()

def parse_args():
    parser = jgtcommon.new_parser("JGT FX MV Trade Stop CLI", "Change stop order of trade by id on FXConnect", "fxmvstop")
    
    parser=jgtcommon.add_demo_flag_argument(parser)
    parser=jgtcommon.add_instrument_standalone_argument(parser,required=False)
    parser=jgtcommon.add_orderid_arguments(parser,required=False)
    parser=jgtcommon.add_tradeid_arguments(parser,required=False)
    parser=jgtcommon.add_stop_arguments(parser,pips_flag=True)
    parser=jgtcommon.add_account_arguments(parser,required=False)
    
    args = jgtcommon.parse_args(parser)

    return args


def change_trade(fx, trade):
    global str_trade_id
    global str_account
    global str_instrument
    global str_stop
    global pips_flag
    global fxtrade,fxtrades
    
    amount = trade.amount
    event = threading.Event()

    offer = Common.get_offer(fx, trade.instrument)

    if not offer:
        raise Exception(
            "The offer is not valid")

    buy = fxcorepy.Constants.BUY
    sell = fxcorepy.Constants.SELL

    buy_sell = sell if trade.buy_sell == buy else buy

    if str_trade_id and trade.trade_id == str_trade_id:
        msg = "Changing Stop for TradeID: {0:s}".format(trade.trade_id)        
        print_jsonl_message(msg,extra_dict={"trade_id":str_trade_id})
    order_id = trade.stop_order_id
    #print("Stop OrderID: {0:s}".format(order_id))
    open_price = trade.open_rate
    last_close_price = trade.close
    amount = trade.amount
    pip_size = offer.PointSize
    #print("Open Price: {0:.5f}".format(open_price))
    #exit(0)
    #@STCIssue WTF is this for ?
    if not pips_flag:
        stopv=str_stop
    else:
        pip_size_str = "{0:.5f}".format(pip_size)
        msg=f"Stop is in PipSized: {pip_size_str} * {str_stop}"
        print(msg)
        if trade.buy_sell == buy:
            stopv = last_close_price-str_stop*pip_size
        else:
            stopv = last_close_price+str_stop*pip_size
    
    future_validation_stop = False
    if future_validation_stop:
        from FXTradeStopValidator import FXTradeMVStopValidator
    
    #Create an object for our outputs
    import FXHelperTransact as fxh
    fxtrade =fxh.trade_row_to_trade_object(trade)
    fxtrade.message=f"Trade stop changing to: {stopv}"
    fxtrade.tojsonfile()
    
    fxtransact_save_prefix_all = "trade_fxmvstop_"
    fxtransact_save_prefix = fxtransact_save_prefix_all+"01_"
    saved_filepath=fxtdh.save_fxtrade_to_file(fxtrade,save_prefix=fxtransact_save_prefix,prefix_to_connection=False,str_order_id=str_trade_id)
    print_jsonl_message("Saved fxtrade",extra_dict={"filepath":saved_filepath})
    fxtrades.add_trade(fxtrade)
    fxtrades.tojsonfile()
    #print(trade)
    #trade = FXTrade.from_string(string)
        
    
    if order_id:
        stop_order_id = trade.stop_order_id
        request = fx.create_order_request(
            order_type=fxcorepy.Constants.Orders.STOP,
            command=fxcorepy.Constants.Commands.EDIT_ORDER,
            OFFER_ID=offer.offer_id,
            ACCOUNT_ID=str_account,
            RATE=stopv,
            TRADE_ID=trade.trade_id,
            ORDER_ID=stop_order_id
        )
    else:
        stop_order_id = trade.stop_order_id
        request = fx.create_order_request(
            order_type=fxcorepy.Constants.Orders.STOP,
            command=fxcorepy.Constants.Commands.CREATE_ORDER,
            OFFER_ID=offer.offer_id,
            ACCOUNT_ID=str_account,
            BUY_SELL=buy_sell,
            RATE=stopv,
            AMOUNT=amount,
            TRADE_ID=trade.trade_id,
            ORDER_ID=stop_order_id
        )

    if request is None:
        raise Exception("Cannot create request")

    def on_changed_order(_, __, order_row):
        nonlocal order_id
        global fxtrade,fxtrades
        if order_row.stop_order_id == order_id:
            msg = "The order has been changed. Order ID: {0:s}".format(
                order_row.trade_id)

            order_changed_data={
                "order_id":order_row.trade_id,
                "new_stop":stopv,
                "trade_id":str_trade_id
            }
            print_jsonl_message(msg,order_changed_data)
            fxtradeupdated=fxh.trade_row_to_trade_object(order_row)
            #print(fxtradeupdated.tojson())
            msg = f"Trade stop order changed to: {stopv}"
            #fxtrade.message=msg
            fxtradeupdated.message=msg
            fxtradeupdated.tojsonfile()
            
            fxtransact_save_prefix = fxtransact_save_prefix_all+"02_"
            written_filepath=fxtdh.save_fxtrade_to_file(fxtrade,save_prefix=fxtransact_save_prefix,prefix_to_connection=False,str_order_id=str_trade_id)
            #@STCGoal Expect that we will have a trades.json with before the change and after the change
            fxtrades.add_trade(fxtradeupdated)
            fxtrades.tojsonfile()
            msg = "We are done saving the trade after it was changed"
            
            print_jsonl_message(msg,extra_dict={"filepath":written_filepath})

    trades_table = fx.get_table(ForexConnect.TRADES)

    trades_listener = Common.subscribe_table_updates(trades_table,
                              on_change_callback=on_changed_order)

    try:
        resp = fx.send_request(request)
        sleep(2)

    except Exception as e:
        common_samples.print_exception(e)
        trades_listener.unsubscribe()
    else:
        # Waiting for an order to appear or timeout (default 30)
        sleep(1)
        trades_listener.unsubscribe()


def on_each_row(fx, row_data):
    global str_instrument,str_trade_id
    trade = None
    if str_instrument and row_data.instrument == str_instrument:
        #print("Changing trad, row_data:")
        #print(row_data)
        change_trade(fx, row_data)
    elif not str_instrument:
        if str_trade_id and row_data.trade_id == str_trade_id:
            #print("Changing trade, row_data:")
            #print(row_data)
            change_trade(fx, row_data)


def check_trades(fx, table_manager, account_id):
    orders_table = table_manager.get_table(ForexConnect.TRADES)
    if len(orders_table) == 0:
        print("There are no trades!")
    else:
        for row in orders_table:
            on_each_row(fx, row)


def main():
    global str_account
    global str_instrument
    global str_stop
    global str_trade_id
    global pips_flag

    args = parse_args()
    quiet=args.quiet
    str_user_id,str_password,str_url, str_connection,str_account = jgtcommon.read_fx_str_from_config(demo=args.demo)
    str_session_id = ""
    str_pin = ""
    
    str_instrument = args.instrument if args.instrument else None
    str_account = args.account
    str_stop = args.stop
    pips_flag=args.pips if args.pips else False
    
    str_trade_id = args.tradeid if args.tradeid else None
    if str_trade_id is None and args.orderid:
        str_trade_id = args.orderid #support using -id
    if str_trade_id is None:
        print("Trade ID must be specified")
        return
    
    event = threading.Event()

    if not str_stop:
        print("Stop level must be specified")
        return


    with ForexConnect() as fx:
        fx.login(str_user_id, str_password, str_url, str_connection, str_session_id,
                 str_pin, common_samples.session_status_changed)
        str_account_fix= str_account if str_connection != "Demo" else None
        account = Common.get_account(fx, str_account_fix)
        #print("Account:")
        #print(account)

        table_manager = fx.table_manager

        if not account:
            raise Exception(
                "The account '{0}' is not valid".format(account))
        else:
            str_account = account.account_id
            msg = "AccountID='{0}'".format(str_account)
            
            print_jsonl_message(msg,extra_dict={"account_id":str_account})

        #offer = Common.get_offer(fx, str_instrument)

        # if not offer:
        #     raise Exception(
        #         "The instrument '{0}' is not valid".format(str_instrument))

        check_trades(fx, table_manager, account.account_id)

        try:
            fx.logout()
        except Exception as e:
            common_samples.print_exception(e)


if __name__ == "__main__":
    main()
    #input("Done! Press enter key to exit\n")



# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.



import argparse
import datetime
import os
import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtutils import jgtconstants as constants

from jgtutils import jgtos, jgtcommon, jgtpov

import re
from urllib.parse import urlsplit
from urllib.request import urlopen

from forexconnect import ForexConnect

import common_samples

def parse_args():
    parser = jgtcommon.new_parser("JGT FX Report CLI", "Obtain a report from FXConnect", "fxreport")
    parser=jgtcommon.add_demo_flag_argument(parser)
    #parser = argparse.ArgumentParser(description='Process command parameters.')
    #common_samples.add_main_arguments(parser)
    common_samples.add_report_date_arguments(parser)
    #args = parser.parse_args()
    args=jgtcommon.parse_args(parser)

    return args


def month_delta(date, delta):
    m, y = (date.month + delta) % 12, date.year + (date.month + delta - 1) // 12
    if not m:
        m = 12
    d = min(date.day, [31, 29 if y % 4 == 0 and not y % 400 == 0 else 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][m-1])
    return date.replace(day=d, month=m, year=y)


def get_reports(fc, dt_from, dt_to):
    accounts_response_reader = fc.get_table_reader(ForexConnect.ACCOUNTS)
    if dt_to is None:
        dt_to = datetime.datetime.today()
    if dt_from is None:
        dt_from = month_delta(datetime.datetime.today(), -1)
    
    for account in accounts_response_reader:
        print("")
        print("Obtaining report URL...")
        url = fc.session.get_report_url(account.account_id, dt_from, dt_to, "html", None)
        
        print("account_id={0:s}; Balance={1:.5f}".format(account.account_id, account.balance))
        print("Report URL={0:s}\n".format(url))
        file_name = os.path.join(os.getcwd(), account.account_id)
        file_name += ".html"
        
        print("Connecting...")
        response = urlopen(url)
        print("OK")
        print("Downloading report...")

        abs_path = '{0.scheme}://{0.netloc}/'.format(urlsplit(url))
        with open(file_name, 'w') as file:
            report = response.read().decode('utf-8')
            report = re.sub(r'((?:src|href)=")[/\\](.*?")', r'\1' + abs_path + r'\2', report)
            file.write(report)
            print("Report is saved to {0:s}\n".format(file_name))


def main():
    args = parse_args()
    str_user_id,str_password,str_url, str_connection,str_account = jgtcommon.read_fx_str_from_config(demo=args.demo)

    str_session_i_d=""
    str_pin=""
    date_from = args.datefrom
    date_to = args.dateto

    with ForexConnect() as fx:
        try:
            fx.login(str_user_id, str_password, str_url, str_connection,
                     str_session_i_d, str_pin, common_samples.session_status_changed)
            
            get_reports(fx, date_from, date_to)

        except Exception as e:
            common_samples.print_exception(e)
        try:
            fx.logout()
        except Exception as e:
            common_samples.print_exception(e)


if __name__ == "__main__":
    main()
    print("")



from typing import Callable, Any
from . import fxcorepy


class EachRowListener(fxcorepy.AO2GEachRowListener):
    """The class implements the abstract class AO2GEachRowListener and calls the passed function on the iteration through rows of a table."""
    def __init__(self, on_each_row_callback: Callable[[fxcorepy.AO2GEachRowListener, str, fxcorepy.O2GRow], None] = None,
                 data: Any = None) -> None:
        """ The constructor.
        
            Parameters
            ----------
            on_each_row_callback : 
                The function that is called on an iteration through rows of a table.
            data : typing.Any
                Any user's object. The default value of the parameter is None.
        
            Returns
            -------
            None
        
        """
        fxcorepy.AO2GEachRowListener.__init__(self)
        self._on_each_row_callback = on_each_row_callback
        self._data = data

    @property
    def data(self) -> Any:
        """ Gets data passed in the constructor.
        
            Returns
            -------
            typing.Any
        
        """
        return self._data

    def on_each_row(self, row_id: str, row_data: fxcorepy.O2GRow) -> None:  # native call
        """ Implements the method AO2GEachRowListener.on_each_row and calls the function that processes notifications on the iteration through the rows of a table. The function is passed in the constructor.
        
            Returns
            -------
            
        
        """
        if self._on_each_row_callback:
            self._on_each_row_callback(self, row_id, row_data)



# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

import os
import sys

from FXHelperTransact import print_jsonl_message

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtutils import jgtconstants as constants

from jgtutils import jgtos, jgtcommon, jgtpov

from forexconnect import ForexConnect, EachRowListener

from jgtutils.FXTransact import FXTransactWrapper
from jgtutils.FXTransact import FXTransactDataHelper as fxtdh

import common_samples 

import json

def parse_args():
    parser = jgtcommon.new_parser("JGT FX Transact CLI", "List and hopefully manage trade and order on FXConnect", "fxtransact")
    
    parser=jgtcommon.add_demo_flag_argument(parser)
    parser=jgtcommon.add_verbose_argument(parser)
    parser=jgtcommon.add_instrument_standalone_argument(parser,required=False)
    parser=jgtcommon.add_orderid_arguments(parser,required=False)
    parser=jgtcommon.add_tradeid_arguments(parser,required=False)
    parser=jgtcommon.add_account_arguments(parser,required=False)
    
    parser.add_argument('-table',
                        metavar="TABLE",
                        default="all",
                        help='The print table. Possible values are: orders - orders table,\
                        trades - trades table. Default value is trades. Optional parameter.')
    # parser.add_argument('-account', metavar="AccountID", required=False,
    #                     help='Account ID')
    
    # parser.add_argument('-id','--orderid', metavar="OrderID", required=False,
    #                     help='The identifier (optional for filtering).')
    #optional instrument
    # parser.add_argument('-i','--instrument', metavar="Instrument", required=False,
    #                     help='The instrument (optional for filtering).')
    #-save
    parser.add_argument('-save','--save', required=False,
                        help='Save the output to a file.', action='store_true')
    
    
    args=jgtcommon.parse_args(parser)

    return args

str_order_id = None
str_trade_id = None
str_instrument = None

def get_account(table_manager, quiet=True):
    accounts_table = table_manager.get_table(ForexConnect.ACCOUNTS)
    for account_row in accounts_table:
        msg = "{" + " \"AccountID\": {0:s}, \"Balance\": {1:.5f} ".format(account_row.account_id, account_row.balance) + "}"
        print(msg)
    return accounts_table.get_row(0)

from jgtutils.FXTransact import FXOrder

def parse_order_row(order_row, account_id,quiet=True):
    global str_order_id, str_instrument
    if order_row.table_type == ForexConnect.ORDERS:
        if not account_id or account_id == order_row.account_id:
            string = _order_row_to_string(order_row)
            #if not quiet:print(string)
            order=FXOrder.from_string(string)
            
            if not str_order_id and not str_instrument:
                json_str = order.tojson()
                if not quiet:print(json_str)
                return order
            
            current_instrument=order.instrument
            
            #try:current_instrument = order.instrument if order.instrument else None 
            #except:pass
            #@STCIssue Contingent OrderID are related to a TradeID, how to manage that ?
            
            if str_instrument and str_instrument == current_instrument:
                json_str = order.tojson()
                if not quiet:print(json_str)
                return order
            
            if str_instrument:
                return None
            
            if not str_order_id : #NO FILTERING
                json_str = order.tojson()
                if not quiet:print(json_str)
                return order
            else:
                if str_order_id == str(order.order_id):
                    json_str = order.tojson()
                    if not quiet:print(json_str)
                    return order
        return None
    return None

def _order_row_to_string(order_row):
    string = ""
    for column in order_row.columns:
        string += column.id + "=" + str(order_row[column.id]) + "; "
    return string

from jgtutils.FXTransact import FXOrders
def parse_orders(table_manager, account_id,quiet=True):
    orders_table = table_manager.get_table(ForexConnect.ORDERS)
    if len(orders_table) == 0:
        if not quiet:print_jsonl_message("Table is empty!")
        return None
    else:
        fxorders:FXOrders=FXOrders()
        for order_row in orders_table:
            order_data=parse_order_row(order_row, account_id)
            if order_data:
                fxorders.add_order(order_data)
        return fxorders


from jgtutils.FXTransact import FXTrade
def parse_trade_row(trade_row, account_id,quiet=True):
    global str_order_id, str_instrument,str_trade_id
    if trade_row.table_type == ForexConnect.TRADES:
        if not account_id or account_id == trade_row.account_id:
            trade_data = {}
            string =_trade_row_to_string(trade_row, trade_data)
            trade = FXTrade.from_string(string)
            
            if str_instrument and str_instrument == trade.instrument:
                json_str = trade.tojson()
                if not quiet:print(json_str)
                return trade
            
            if str_instrument:
                return None
                
            if not str_order_id and not str_trade_id: #NO FILTERING
                json_str = trade.tojson()
                if not quiet:print(json_str)
                return trade
            else:
                cur_id:str = str(trade.trade_id)
                #by tradeid
                if str_trade_id and str_trade_id == str(cur_id):
                    json_str = trade.tojson()
                    if not quiet:print(json_str)
                    return trade
                #by orderid #@STCIssue Might Want to support filtering by orderid of the original order id that generated the trade
                if str_order_id and str_order_id == str(cur_id):
                    json_str = trade.tojson()
                    if not quiet:print(json_str)
                    return trade

def _trade_row_to_string(trade_row, trade_data):
    string=""
    for column in trade_row.columns:
        string += column.id + "=" + str(trade_row[column.id]) + "; "
        trade_data[column.id] = trade_row[column.id]
    return string

from jgtutils.FXTransact import FXTrades

def parse_trades(table_manager, account_id,quiet=True)->FXTrades:
    trades_table = table_manager.get_table(ForexConnect.TRADES)
    if len(trades_table) == 0:
        if not quiet:print_jsonl_message("Table is empty!")
        return None
    else:
        trades=FXTrades()
        for trade_row in trades_table:
            trade_data=parse_trade_row(trade_row, account_id)
            if trade_data:
                trades.add_trade(trade_data)
        return trades


def main():
    global str_order_id, str_instrument,str_trade_id
    args = parse_args()
    quiet=args.quiet
    str_user_id,str_password,str_url, str_connection,str_account = jgtcommon.read_fx_str_from_config(demo=args.demo)
    str_session_id = ""
    str_pin = ""
    str_order_id=args.orderid if args.orderid else None
    str_trade_id=args.tradeid if args.tradeid else None
    if str_trade_id and not str_order_id:
        str_order_id=str_trade_id #@STCIssue Fix not working for trades
    str_instrument=args.instrument if args.instrument else None
    save_flag=True if args.save else False
    
    str_table = args.table

    if str_table != 'orders' and  str_table != 'trades' :
        str_table = 'all'

    with ForexConnect() as fx:

        fx.login(str_user_id, str_password, str_url,
                 str_connection, str_session_id, str_pin,
                 common_samples.session_status_changed)

        table_manager = fx.table_manager
        account = get_account(table_manager)

        if not account:
            raise Exception("No valid accounts")

        fxtransactwrapper = FXTransactWrapper()
        
        fxorders:FXOrders =None
        if str_table == "orders" or str_table == "all":
            fxorders:FXOrders = parse_orders(table_manager, account.account_id)
            if fxorders:
                if not quiet:print(fxorders.tojson())
                fxtransactwrapper.add_orders(fxorders)
        
        fxtrades:FXTrades =None
        if str_table == "trades" or str_table == "all":
            fxtrades:FXTrades =parse_trades(table_manager, account.account_id)
            if fxtrades:
                if not quiet:print(fxtrades.tojson())
                fxtransactwrapper.add_trades(fxtrades)

        if save_flag:
            saved_filepath=fxtdh.save_fxtransact_to_file(fxtransactwrapper,str_table,str_connection,save_prefix="fxtransact_",prefix_to_connection=False,str_order_id=str_order_id,str_instrument=str_instrument,str_trade_id=str_trade_id)
            print_jsonl_message("Saved to file",extra_dict={"file":saved_filepath})

        else:# we print the data
            print(fxtransactwrapper.tojson())
            
            
        
        
        try:
            fx.logout()
        except Exception as e:
            common_samples.print_exception(e)


if __name__ == "__main__":
    main()



# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import json
import threading
from time import sleep

import os
import sys

from FXHelperTransact import print_jsonl_message

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from jgtutils import jgtconstants as constants
from jgtutils.jgterrorcodes import ORDER_NOT_FOUND_EXIT_ERROR_CODE

from jgtutils import jgtos, jgtcommon, jgtpov

from forexconnect import fxcorepy, ForexConnect, Common

import common_samples


RAISE_EXCEPTION_ON_ORDER_NOT_FOUND = False



def parse_args():
    parser = jgtcommon.new_parser("JGT FX RemoveEntry Order CLI", "Remove an Entry order on FXConnect", "fxrmorder")
    
    parser=jgtcommon.add_demo_flag_argument(parser)

    # specific arguments
    parser=jgtcommon.add_orderid_arguments(parser)

    args=jgtcommon.parse_args(parser)

    return args


class OrdersMonitor:
    def __init__(self):
        self.__order_id = None
        self.__deleted_orders = {}
        self.__event = threading.Event()

    def on_delete_order(self, _, __, order_row):
        order_id = order_row.order_id
        self.__deleted_orders[order_id] = order_row
        if self.__order_id == order_id:
            self.__event.set()

    def wait(self, time, order_id):
        self.__order_id = order_id

        order_row = self.find_order(order_id)
        if order_row is not None:
            return order_row

        self.__event.wait(time)

        return self.find_order(order_id)

    def find_order(self, order_id):
        if order_id in self.__deleted_orders:
            return self.__deleted_orders[order_id]
        else:
            return None

    def reset(self):
        self.__order_id = None
        self.__deleted_orders.clear()
        self.__event.clear()


def main():
    args = parse_args()
    str_user_id,str_password,str_url, str_connection,str_account = jgtcommon.read_fx_str_from_config(demo=args.demo)
    str_session_id = ""
    str_pin = ""
    str_old = args.orderid

    with ForexConnect() as fx:
        try:
            fx.login(str_user_id, str_password, str_url, str_connection, str_session_id,
                     str_pin, common_samples.session_status_changed)

            order_id = str_old
            orders_table = fx.get_table(ForexConnect.ORDERS)
            orders = orders_table.get_rows_by_column_value("order_id", order_id)
            order = None
            
            for order_row in orders:
                order = order_row
                break

            if order is None:
                msg = "Order {0} not found".format(order_id)
                order_not_found={
                    "order_id":order_id,
                    "status":"not found"
                }
                print_jsonl_message(msg,order_not_found)
                
                if RAISE_EXCEPTION_ON_ORDER_NOT_FOUND:
                    raise Exception(msg)
                else: #Exit the program with an error code 
                    sys.exit(ORDER_NOT_FOUND_EXIT_ERROR_CODE)

            order_account_id = order.account_id
            request = fx.create_request({

                fxcorepy.O2GRequestParamsEnum.COMMAND: fxcorepy.Constants.Commands.DELETE_ORDER,
                fxcorepy.O2GRequestParamsEnum.ACCOUNT_ID: order_account_id,
                fxcorepy.O2GRequestParamsEnum.ORDER_ID: str_old
            })

            orders_monitor = OrdersMonitor()

            orders_table = fx.get_table(ForexConnect.ORDERS)
            orders_listener = Common.subscribe_table_updates(orders_table, on_delete_callback=orders_monitor.on_delete_order)

            try:
                fx.send_request(request)

            except Exception as e:
                common_samples.print_exception(e)
                orders_listener.unsubscribe()
            else:
                # Waiting for an order to delete or timeout (default 30)
                is_deleted = orders_monitor.wait(30, order_id)
                if not is_deleted:
                    msg = "Response waiting timeout expired.\n"
                    timed_out={
                        "order_id":order_id,
                        "status":"waiting timeout expired"
                    }
                    print_jsonl_message(msg,timed_out)
                else:
                    msg="The order has been deleted. Order ID: {0:s}".format(order_row.order_id)
                    order_deleted={
                        "order_id":order_row.order_id,
                        "buy_sell":order_row.buy_sell,
                        "rate":order_row.rate,
                        "status":"deleted"
                                   }
                    print_jsonl_message(msg,order_deleted)
                    sleep(1)
                orders_listener.unsubscribe()

        except Exception as e:
            common_samples.print_exception(e)
        try:
            fx.logout()
        except Exception as e:
            common_samples.print_exception(e)



if __name__ == "__main__":
    main()
    
    #input("Done! Press enter key to exit\n")



# Copyright 2019 Gehtsoft USA LLC

# Licensed under the license derived from the Apache License, Version 2.0 (the "License"); 
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at

# http://fxcodebase.com/licenses/open-source/license.html

# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import traceback

from forexconnect import ForexConnect, Common

from common_samples import OrderMonitor


class TableListenerContainer:
    __response_listener = None
    __request_id = ""
    __order_monitor = None

    def __init__(self, response_listener, fx):
        self.__response_listener = response_listener
        self._fx = fx
        self._listeners = []

    def set_request_id(self, request_id):
        self.__request_id = request_id

    def _on_added_orders(self, listener, row_id, order_row):
        del listener, row_id
        if self.__request_id == order_row.request_id:
            if OrderMonitor.is_closing_order(
                    order_row) or OrderMonitor.is_opening_order(
                        order_row) and self.__order_monitor is None:
                print(
                    "The order has been added. Order ID: {0:s}, Rate: {1:.5f}, Time In Force: {2:s}".format(
                        order_row.order_id, order_row.rate,
                        order_row.time_in_force))
                self.__order_monitor = OrderMonitor(order_row)

    def _on_added_trades(self, listener, row_id, trade_row):
        del listener, row_id
        if self.__order_monitor is not None:
            self.__order_monitor.on_trade_added(trade_row)
            if self.__order_monitor.is_order_completed:
                self._print_result()
                self.__response_listener.stop_waiting()

    def _on_added_closed_trades(self, listener, row_id, closed_trade_row):
        del listener, row_id
        if self.__order_monitor is not None:
            self.__order_monitor.on_closed_trade_added(closed_trade_row)
            if self.__order_monitor.is_order_completed:
                self._print_result()
                self.__response_listener.stop_waiting()

    def _on_added_messages(self, listener, row_id, message_row):
        del listener, row_id
        if self.__order_monitor is not None:
            self.__order_monitor.on_message_added(message_row)
            if self.__order_monitor.is_order_completed:
                self._print_result()
                self.__response_listener.stop_waiting()

    def _on_deleted_orders(self, listener, row_id, row_data):
        del listener, row_id
        order_row = row_data
        if self.__request_id == order_row.request_id:
            if self.__order_monitor is not None:
                print("The order has been deleted. Order ID: {0}".format(
                    order_row.order_id))
                self.__order_monitor.on_order_deleted(order_row)
                if self.__order_monitor.is_order_completed:
                    self._print_result()
                    self.__response_listener.stop_waiting()

    def _print_result_canceled(self, order_id, trades, closed_trades):
        if len(trades) > 0:
            self._print_trades(trades, order_id)
            self._print_closed_trades(closed_trades, order_id)
            print("A part of the order has been canceled. Amount = {0}".format(
                self.__order_monitor.reject_amount))
        else:
            print("The order: OrderID = {0}  has been canceled".format(
                order_id))
            print("The cancel amount = {0}".format(
                self.__order_monitor.reject_amount))

    def _print_result_fully_rejected(self, order_id, trades, closed_trades):
        del trades, closed_trades
        print("The order has been rejected. OrderID = {0}".format(
            order_id))
        print("The rejected amount = {0}".format(
            self.__order_monitor.reject_amount))
        print("Rejection cause: {0}".format(
            self.__order_monitor.reject_message))

    def _print_result_partial_rejected(self, order_id, trades, closed_trades):
        self._print_trades(trades, order_id)
        self._print_closed_trades(closed_trades, order_id)
        print("A part of the order has been rejected. Amount = {0}".format(
            self.__order_monitor.reject_amount))
        print("Rejection cause: {0} ".format(
            self.__order_monitor.reject_message))

    def _print_result_executed(self, order_id, trades, closed_trades):
        self._print_trades(trades, order_id)
        self._print_closed_trades(closed_trades, order_id)

    def _print_result(self):
        if self.__order_monitor is not None:
            result = self.__order_monitor.result
            order = self.__order_monitor.order_row
            order_id = order.order_id
            trades = self.__order_monitor.trade_rows
            closed_trades = self.__order_monitor.closed_trade_rows

            print_result_func = {
                OrderMonitor.ExecutionResult.CANCELED: self._print_result_canceled,
                OrderMonitor.ExecutionResult.FULLY_REJECTED: self._print_result_fully_rejected,
                OrderMonitor.ExecutionResult.PARTIAL_REJECTED: self._print_result_partial_rejected,
                OrderMonitor.ExecutionResult.EXECUTED: self._print_result_executed
            }
            try:
                print_result_func[result](order_id, trades, closed_trades)
            except KeyError:
                pass
            except Exception as e:
                print("Exception: {0}\n".format(e))
                print(traceback.format_exc())

    @staticmethod
    def _print_trades(trades, order_id):
        if len(trades) == 0:
            return
        print(
            "For the order: OrderID = {0} the following positions have been opened:".format(
                order_id))

        for trade in trades:
            trade_id = trade.trade_id
            amount = trade.amount
            rate = trade.open_rate
            print(
                "Trade ID: {0:s}; Amount: {1:d}; Rate: {2:.5f}".format(trade_id,
                                                                       amount,
                                                                       rate))

    @staticmethod
    def _print_closed_trades(closed_trades, order_id):
        if len(closed_trades) == 0:
            return
        print(
            "For the order: OrderID = {0} the following positions have been closed: ".format(
                order_id))

        for closed_trade in closed_trades:
            trade_id = closed_trade.trade_id
            amount = closed_trade.amount
            rate = closed_trade.close_rate
            print(
                "Closed Trade ID: {0:s}; Amount: {1:d}; Closed Rate: {2:.5f}".format(
                    trade_id, amount, rate))

    def subscribe_events(self):
        orders_table = self._fx.get_table(ForexConnect.ORDERS)
        orders_table_listener = Common.subscribe_table_updates(orders_table,
                                                               on_add_callback=self._on_added_orders,
                                                               on_delete_callback=self._on_deleted_orders)
        self._listeners.append(orders_table_listener)

        trades_table = self._fx.get_table(ForexConnect.TRADES)
        trades_table_listener = Common.subscribe_table_updates(trades_table,
                                                               on_add_callback=self._on_added_trades)
        self._listeners.append(trades_table_listener)

        messages_table = self._fx.get_table(ForexConnect.MESSAGES)
        messages_table_listener = Common.subscribe_table_updates(messages_table,
                                                                 on_add_callback=self._on_added_messages)
        self._listeners.append(messages_table_listener)

        closed_trades_table = self._fx.get_table(ForexConnect.CLOSED_TRADES)
        closed_trades_table_listener = Common.subscribe_table_updates(closed_trades_table,
                                                                      on_add_callback=self._on_added_closed_trades)
        self._listeners.append(closed_trades_table_listener)

    def unsubscribe_events(self):
        for listener in self._listeners:
            listener.unsubscribe()
        self._listeners = []
